// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  class SceneItem;

  class SceneComponent : std::enable_shared_from_this<SceneComponent> {
    private:
      std::weak_ptr<SceneItem> item;

    protected:
      /**
       * Custom component listener that is invoked when the component is meant
       * to initialize.
       */
      virtual void onInit() = 0;

    public:
      /**
       * Initializes this scene component.
       * 
       * @param item Scene item that this component belongs to.
       */
      void init(const std::shared_ptr<SceneItem> item);

      /**
       * Returns the scene item that this scene component belongs to.
       * 
       * @return Reference to the scene item that this component belongs to.
       */
      std::shared_ptr<SceneItem> getItem();
  };
}