// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/item/SceneItemTransform.hpp"
#include "scene/item/SceneItemComponents.hpp"

namespace Dawn {
  class Scene;

  class SceneItem final :
    public SceneItemTransform,
    public SceneItemComponents,
    public std::enable_shared_from_this<SceneItem>
  {
    private:
      std::weak_ptr<Scene> scene;

    protected:
      std::shared_ptr<SceneItem> sceneItemComponentsSelf() override;

    public:
      /**
       * Constructs a scene item.
       * 
       * @param scene Scene that this item belongs to.
       */
      SceneItem(const std::weak_ptr<Scene> scene);

      /**
       * Returns the scene that this scene item belongs to.
       * 
       * @return Pointer to the scene that this item belongs to.
       */
      std::shared_ptr<Scene> getScene();

      /**
       * Returns the scene that this item belongs to.
       * 
       * @return Scene that this item belongs to.
       */
      virtual ~SceneItem();
  };
}